import os
from typing import Any, Dict, List, Union

from pyadvtools import write_list
from pybibtexer.bib.bibtexparser import Library
from pybibtexer.main import PythonRunBib, PythonWriters

from ...utils.utils import html_head, html_style, html_tail, textarea_header, textarea_tail


def generate_html_content(html_body, abbr_standard):
    """Create complete HTML document from body content.

    Args:
        html_body: List of HTML body content lines.
        abbr_standard (str): Standard abbreviation for the document.

    Returns:
        List[str]: Complete HTML document as list of lines.
    """
    return [html_head.format(abbr_standard), html_style, "\n"] + html_body + [html_tail]


def generate_html_from_bib_data(
    abbr_standard: str,
    original_bib_data: Union[List[str], str, Library],
    path_output: str,
    options: Dict[str, Any] = {},
) -> List[str]:
    """Generate HTML from bibliography data.

    Args:
        abbr_standard (str): Standard abbreviation for the publication.
        original_bib_data (Union[List[str], str, Library]): Bibliography data in various formats.
        path_output (str): Path to output directory.
        options (Dict[str, Any], optional): Additional processing options. Defaults to {}.
        full_json_c (str, optional): Path to conferences JSON file. Defaults to "".
        full_json_j (str, optional): Path to journals JSON file. Defaults to "".

    Returns:
        List[str]: List of HTML body content lines.
    """
    # Set processing options
    processing_options: dict = {
        # convert_str_to_library
        "is_standardize_bib": False,
        # middlewares_str_to_library.py
        "is_display_implicit_comments": False,
        #
        # convert_library_to_library.py
        # middlewares_library_to_library.py
        "function_common_again": False,
        "function_common_again_abbr": False,
        "function_common_again_zotero": False,
        "function_common_again_save": False,
        "abbr_index_article_for_abbr": 2,
        "abbr_index_inproceedings_for_abbr": 2,
        #
        # convert_library_to_str.py
        "empty_entry_cite_keys": True,
        # middlewares_library_to_str.py
        "is_sort_entry_fields": True,
        "is_sort_blocks": True,
        "sort_entries_by_field_keys_reverse": True,
    }
    # Update with provided options
    processing_options.update(options)

    # Process bibliography data
    _python_bib = PythonRunBib(processing_options)
    _, zotero_library, _ = _python_bib.parse_to_multi_standard_library(original_bib_data)

    _python_writer = PythonWriters(processing_options)

    # Generate HTML content for each entry
    html_body = []
    for entry in zotero_library.entries:
        html_body.append(_format_entry_to_html(entry, abbr_standard, _python_writer.write_to_str([entry])))

    # Create complete HTML document if entries exist
    if len(html_body) > 0:
        html_body = (
            [f'<h2 id="{abbr_standard.lower()}">{abbr_standard} - {len(zotero_library.entries)}</h2>\n', "<ul>\n"]
            + html_body
            + ["</ul>\n"]
        )

        html_content = generate_html_content(html_body, abbr_standard)
        output_dir = os.path.join(path_output, abbr_standard)

        # Write output file
        write_list(html_content, f"{abbr_standard}.html", "w", output_dir, False)

    return html_body


def _format_entry_to_html(entry, abbr, data_list):
    """Format a single bibliography entry into HTML.

    Args:
        entry: Bibliography entry dictionary.
        abbr (str): Publication abbreviation.
        data_list: List of formatted bibliography data.

    Returns:
        str: HTML formatted entry string.
    """
    # Extract entry fields
    number = entry["number"] if "number" in entry else ""
    pages = entry["pages"] if "pages" in entry else ""
    title = entry["title"] if "title" in entry else ""
    year = entry["year"] if "year" in entry else ""
    volume = entry["volume"] if "volume" in entry else ""

    # Get URL (DOI preferred, fall back to URL)
    url = ""
    if "doi" in entry:
        doi = entry["doi"]
        url = doi if doi.startswith("http") else f"https://doi.org/{doi}"
    elif "url" in entry:
        url = entry["url"]

    # Format entry in APA style
    line = _format_entry_to_apa_style(title, year, volume, number, pages, url, abbr)

    line = f"<li><details>\n<summary>\n{line.strip()}\n</summary>\n"

    # Create HTML structure with details
    return line + textarea_header + "".join(data_list).rstrip() + textarea_tail + "\n</details></li>\n"


def _format_entry_to_apa_style(title, year, volume, number, pages, url, abbr):
    """Format entry in APA citation style.

    Args:
        title (str): Article title.
        year (str): Publication year.
        volume (str): Journal volume.
        number (str): Issue number.
        pages (str): Page numbers.
        url (str): Article URL.
        abbr (str): Publication abbreviation.

    Returns:
        str: APA formatted citation string.
    """
    line = f"({year}). {title}. <em>{abbr}</em>"

    if volume:
        line += f", <em>{volume}</em>"
        if number:
            line += f"({number})"

    if pages:
        line += f", {pages}"

    line += "."

    if url:
        line += f" (<a href='{url}'>www</a>)"

    return line



"""Main module for PyEasyPhD core functionality.

This module contains the core classes for processing academic papers,
managing bibliographies, and converting between different formats.
"""

__all__ = ["BasicInput", "PandocMdTo", "PythonRunMd", "PythonRunTex"]

from .basic_input import BasicInput
from .pandoc_md_to import PandocMdTo
from .python_run_md import PythonRunMd
from .python_run_tex import PythonRunTex



"""Tools module for PyEasyPhD advanced functionality.

This module provides advanced tools for bibliography processing,
search functionality, and content generation.
"""

__all__ = ["PyRunBibMdTex", "Searchkeywords", "generate_from_bibs_and_write", "PaperLinksGenerator"]

from .generate.generate_from_bibs import generate_from_bibs_and_write
from .generate.generate_links import PaperLinksGenerator
from .py_run_bib_md_tex import PyRunBibMdTex
from .search.search_keywords import Searchkeywords



# coding=utf-8

import json
import os
from datetime import datetime
from typing import Dict, List

from pyadvtools import standard_path


class PaperLinksGenerator(object):
    """Generate markdown files with paper links from JSON data."""

    def __init__(
        self,
        full_json_c: str,
        full_json_j: str,
        full_json_k: str,
        data_base_path: str,
        keywords_category_name: str = "",
        display_year_period: int = 10,
    ):
        """Initialize the generator with base paths.

        Args:
            full_json_c (str): Path to conferences JSON file.
            full_json_j (str): Path to journals JSON file.
            full_json_k (str): Path to keywords JSON file.
            data_base_path (str): Path to data files directory.
            keywords_category_name (str, optional): Category name for keywords. Defaults to "".
            display_year_period (int, optional): Number of years to display. Defaults to 10.
        """
        self.full_json_c = full_json_c
        self.full_json_j = full_json_j
        self.full_json_k = full_json_k

        self.data_base_path = standard_path(data_base_path)

        # Process keyword category name and load data
        keywords_category_name = keywords_category_name.lower().strip() if keywords_category_name else ""
        category_prefix = f"{keywords_category_name}_" if keywords_category_name else ""
        keywords_list = self._load_json_data("keywords").get(f"{category_prefix}keywords", [])

        # Validate data availability
        if not keywords_list or not keywords_category_name:
            keywords_list, keywords_category_name = [], ""

        self.keywords_category_name = keywords_category_name
        self.keywords_list: List[str] = keywords_list

        self.display_year_period = display_year_period

    def generate_yearly_links(self, cj: str, folder_name=os.path.join("data", "Yearly")) -> None:
        """Generate yearly markdown table with paper links.

        Args:
            cj (str): Publication type - 'conferences' or 'journals'.
            folder_name (str, optional): Output folder name. Defaults to "data/Yearly".
        """
        flags = self._get_yearly_flags(cj)
        folder_flags = [f"{f}_all_months" for f in flags]

        self._generate_links(cj, flags, folder_flags, folder_name)

    def generate_monthly_links(self, folder_name=os.path.join("data", "Monthly")) -> None:
        """Generate monthly markdown table with journal paper links.

        Args:
            folder_name (str, optional): Output folder name. Defaults to "data/Weekly".
        """
        cj = "Journals"

        flags = ["All Months"]
        folder_flags = [f"current_year_{f.replace(' ', '_').lower()}" for f in flags]

        self._generate_links(cj, flags, folder_flags, folder_name)

    def generate_weekly_links(self, folder_name=os.path.join("data", "Weekly")) -> None:
        """Generate weekly markdown table with journal paper links.

        Args:
            folder_name (str, optional): Output folder name. Defaults to "data/Weekly".
        """
        cj = "Journals"

        flags = ["Current Issue", "Current Month"]
        folder_flags = [f"current_year_{f.replace(' ', '_').lower()}" for f in flags]

        self._generate_links(cj, flags, folder_flags, folder_name)

    def _generate_links(self, cj, flags, folder_flags, folder_name) -> None:
        json_data = self._load_json_data(cj.lower())
        if not json_data:
            return None

        # publisher
        md_content = self._create_md_header_publisher(cj, flags)
        table_rows = self._generate_table_rows_publisher(json_data, cj, folder_flags, folder_name)
        if table_rows:
            md_content.extend(table_rows)
            self._write_md_file(md_content, folder_name, f"{cj}_Publisher.md")

        # abbr
        md_content = self._create_md_header_abbr(cj, flags)
        table_rows = self._generate_table_rows_abbr(json_data, cj, folder_flags, folder_name)
        if table_rows:
            md_content.extend(table_rows)
            self._write_md_file(md_content, folder_name, f"{cj}_Abbreviation.md")
            self._convert_md_to_html(folder_name, f"{cj}_Abbreviation")

        return None

    def _convert_md_to_html(self, folder_name, file_name):
        """Convert markdown file to HTML using pandoc."""
        base_path = os.path.join(self.data_base_path, f"{folder_name}")
        file_md = os.path.join(base_path, f"{file_name}.md")
        file_html = os.path.join(base_path, f"{file_name}.html")

        try:
            os.system(f"pandoc {file_md} -o {file_html}")
            os.remove(file_md)
        except Exception as e:
            print(f"Pandoc conversion error: {e}")

    def generate_ieee_early_access_links(self, folder_name=os.path.join("data", "Weekly")) -> None:
        """Generate markdown for IEEE Early Access papers."""
        md_content = [
            "# Papers from Early Access\n\n",
            "|Publisher|**Current Month Papers**|**All Papers**|\n",
            "|-|-|-|\n",
        ]

        links = self._get_ieee_links()
        if any(links):
            md_content.append(f"|IEEE|{links[0]}|{links[1]}|\n")
            self._write_md_file(md_content, folder_name, "Journals_Early_Access.md")

        return None

    def _load_json_data(self, file_name: str) -> Dict:
        """Load JSON data from file."""
        try:
            if file_name.lower().strip() == "conferences":
                file_path = os.path.expanduser(self.full_json_c)
            elif file_name.lower().strip() == "journals":
                file_path = os.path.expanduser(self.full_json_j)
            elif file_name.lower().strip() == "keywords":
                file_path = os.path.expanduser(self.full_json_k)
            else:
                file_path = ""

            if not os.path.exists(file_path):
                return {}

            with open(file_path, "r", encoding="utf-8") as file:
                return json.load(file)

        except Exception as e:
            print(f"Error loading {file_name}.json: {e}")
            return {}

    def _get_yearly_flags(self, cj: str) -> List[str]:
        """Get yearly flags based on publication type."""
        current_year = datetime.now().year
        years = [str(y) for y in range(current_year - self.display_year_period, current_year)]
        flags = sorted(years, reverse=True)

        if cj.lower() == "conferences":
            flags = [str(current_year)] + flags

        return flags

    def _create_md_header_publisher(self, cj: str, flags: List[str]) -> List[str]:
        """Create markdown table header."""
        return [
            f"# Papers from {cj.title()} of Different Publishers\n\n",
            f"| | {'|'.join(f'**{f}**' for f in flags)}|\n",
            f"|-|{'|'.join('-' for _ in flags)}|\n",
        ]

    def _create_md_header_abbr(self, cj: str, flags: List[str]) -> List[str]:
        """Create markdown table header."""
        return [
            f"# Papers from {cj.title()} of Different Publishers\n\n",
            f"| |Publishers|{'|'.join(f'**{f}**' for f in flags)}|\n",
            f"|-|-|{'|'.join('-' for _ in flags)}|\n",
        ]

    # publisher
    def _generate_table_rows_publisher(
        self, json_data: Dict, cj: str, folder_flags: List[str], period: str
    ) -> List[str]:
        """Generate markdown table rows."""
        rows = []
        idx = 1

        for publisher in json_data:
            cells = self._get_link_cells_publisher(publisher, cj, folder_flags, period)
            if any(cells):
                rows.append(f"|{idx}|{'|'.join(cells)}|\n")
                idx += 1

        return rows

    def _get_link_cells_publisher(self, publisher: str, cj: str, folder_flags: List[str], period: str) -> List[str]:
        """Get link cells for a publisher."""
        cells = []

        for flag in folder_flags:
            link_path = os.path.join(period, cj, flag, publisher.lower(), f"{publisher.lower()}_link.html")
            full_path = os.path.join(self.data_base_path, link_path)

            if os.path.exists(full_path):
                cells.append(f"[{publisher}]({link_path})")
            else:
                cells.append("")

        return cells

    # abbr
    def _generate_table_rows_abbr(self, json_data: Dict, cj: str, folder_flags: List[str], period: str) -> List[str]:
        """Generate markdown table rows."""
        rows = []
        idx = 1

        for publisher in json_data:
            if cj.lower() not in json_data[publisher]:
                continue

            for abbr in json_data[publisher][cj.lower()]:
                cells = self._get_link_cells_abbr(publisher, abbr, cj, folder_flags, period)
                if any(cells):
                    rows.append(f"|{idx}|{publisher}|{'|'.join(cells)}|\n")
                    idx += 1

        return rows

    def _get_link_cells_abbr(
        self, publisher: str, abbr: str, cj: str, folder_flags: List[str], period: str
    ) -> List[str]:
        """Get link cells for a abbr."""
        cells = []
        for flag in folder_flags:
            link_path = os.path.join(period, cj, flag, publisher.lower(), abbr, f"{abbr}.html")
            full_path = os.path.join(self.data_base_path, link_path)
            if os.path.exists(full_path):
                cells.append(f"[{abbr}]({link_path})")
            else:
                cells.append("")

        return cells

    def _get_ieee_links(self, folder_name=os.path.join("data", "Weekly")) -> List[str]:
        """Get IEEE Early Access links."""
        links = []
        link_paths = [
            os.path.join(folder_name, "Journals_Early_Access", "current_year_current_month", "ieee", "ieee_link.html"),
            os.path.join(folder_name, "Journals_Early_Access", "all_years_all_months", "ieee", "ieee_link.html"),
        ]

        for link_path in link_paths:
            full_path = os.path.join(self.data_base_path, link_path)
            if os.path.exists(full_path):
                links.append(f"[IEEE Early Access]({link_path})")
            else:
                links.append("")

        return links

    def _write_md_file(self, content: List[str], period: str, file_name: str) -> None:
        """Write markdown content to file."""
        if len(content) == 0:
            return None

        output_dir = os.path.join(self.data_base_path, period)
        os.makedirs(output_dir, exist_ok=True)

        output_file = os.path.join(output_dir, file_name)
        with open(output_file, "w") as f:
            f.writelines(content)
        print(f"Generated: {output_file}")

        return None

    def generate_keywords_links_weekly(self, cj: str, folder_name=os.path.join("data", "Weekly")):
        flags = ["Current Issue", "Current Month"]
        folder_flags = [f"current_year_{f.replace(' ', '_').lower()}" for f in flags]

        self._generate_keywords_links(cj, folder_name, flags, folder_flags)

    def generate_keywords_links_monthly(self, cj: str, folder_name=os.path.join("data", "Monthly")):
        flags = ["All Months"]
        folder_flags = [f"current_year_{f.replace(' ', '_').lower()}" for f in flags]

        self._generate_keywords_links(cj, folder_name, flags, folder_flags)

    def generate_keywords_links_yearly(self, cj: str, folder_name=os.path.join("data", "Yearly")):
        flags = self._get_yearly_flags(cj)
        folder_flags = [f"{f}_all_months" for f in flags]

        self._generate_keywords_links(cj, folder_name, flags, folder_flags)

    def _generate_keywords_links(self, cj: str, folder_name: str, flags: List[str], folder_flags: List[str]):
        json_data = self._load_json_data(cj.title())
        if not json_data:
            return None

        keyword_publisher_abbr = self._process_keywords(cj, json_data)

        all_data_list = ["# Keywords\n\n", "| |Keywords|Links|\n", "|-|-|-|\n"]
        idx = 1
        for keyword in self._default_or_customized_keywords(keyword_publisher_abbr):
            data_list = [
                f"# {keyword.title()}\n\n",
                "|Publishers|Abbreviations|" + "|".join(flags) + "|\n",
                "|-|-|" + "|".join(["-" for _ in flags]) + "|\n",
            ]

            for publisher in keyword_publisher_abbr[keyword]:
                for abbr in keyword_publisher_abbr[keyword][publisher]:
                    lines = []
                    for ff in folder_flags:
                        ll = os.path.join(folder_name, cj.title(), ff, publisher.lower(), abbr, f'{abbr}.html')
                        if os.path.exists(os.path.join(self.data_base_path, ll)):
                            lines.append(f"[Link]({ll})")
                        else:
                            lines.append("")

                    if any(lines):
                        data_list.append(f"|{publisher}|{abbr}|" + "|".join(lines) + "|\n")

            if len(data_list) == 3:
                continue

            self._write_md_file(
                data_list, os.path.join(folder_name, f"{cj.title()}_Keywords"), f"{keyword.replace(' ', '_')}.md"
            )

            # Pandoc
            self._convert_md_to_html_keyword(folder_name, cj, keyword)

            ll = os.path.join(folder_name, f"{cj.title()}_Keywords", f"{keyword.replace(' ', '_')}.html")
            all_data_list.append(f"|{idx}|{keyword}|[Link]({ll})|\n")

            idx += 1

        category_postfix = f"_{self.keywords_category_name.title()}" if self.keywords_category_name else ""
        self._write_md_file(all_data_list, f"{folder_name}", f"{cj.title()}_Keywords{category_postfix}.md")

    def _default_or_customized_keywords(self, json_data):
        keywords = list(json_data.keys())

        # Get and sort publication types
        if self.keywords_category_name and self.keywords_list:
            _keywords = []
            for keyword in self.keywords_list:
                if keyword in keywords:
                    _keywords.append(keyword)
            return _keywords
        else:
            # default
            return sorted(keywords)

    def _convert_md_to_html_keyword(self, folder_name, cj, keyword):
        """Convert markdown file to HTML using pandoc."""
        base_path = os.path.join(self.data_base_path, folder_name, f"{cj.title()}_Keywords")
        file_md = os.path.join(base_path, f"{keyword.replace(' ', '_')}.md")
        file_html = os.path.join(base_path, f"{keyword.replace(' ', '_')}.html")

        try:
            os.system(f"pandoc {file_md} -o {file_html}")
            os.remove(file_md)
        except Exception as e:
            print(f"Pandoc conversion error: {e}")

    def _process_keywords(self, cj: str, json_data: dict):
        keyword_publisher_abbr = {}

        for publisher in json_data:
            for abbr in json_data[publisher][cj.lower()]:

                keywords_dict = json_data[publisher][cj.lower()][abbr].get("keywords_dict", {})

                # Clean and sort keywords
                cleaned_keywords = {}
                for category, words in keywords_dict.items():
                    if category.strip():
                        sorted_words = sorted(set([word.strip() for word in words if word.strip()]))
                        cleaned_keywords[category.strip()] = sorted_words

                # For category
                # Flatten keywords and remove duplicates
                all_keywords = []
                for category, words in cleaned_keywords.items():
                    all_keywords.extend(words)
                    all_keywords.append(category)
                all_keywords = sorted(set(all_keywords))

                for keyword in all_keywords:
                    keyword_publisher_abbr.setdefault(keyword, {}).setdefault(publisher, []).append(abbr)

        return keyword_publisher_abbr



import datetime
import os
import re

from pyadvtools import combine_content_in_list, read_list, write_list

html_head = """<!DOCTYPE html>
<html>
<head>
  <meta charset="utf-8">
  <title>{}</title>
"""

html_style = """  <style>
    html {font-size: 22px;}
    body {margin: 0 auto; max-width: 76em;}
    #copyID {font-size: 18px;}
  </style>
  <script>
    function copy(element) {
      if (element.type == "button"){
      element.type="text";
      }
      element.style.color="black";
      element.style.backgroundColor="#C7EDCC";
      element.select();
      element.setSelectionRange(0, 99999);
      navigator.clipboard.writeText(element.value);
      window.getSelection().removeAllRanges();
      element.type="button";
    }
  </script>
</head>
<body>
"""

html_tail = """
</body>
</html>
"""

textarea_header = '<textarea id="copyID" onclick="copy(this)" rows="16" cols="145">\n'
textarea_tail = "\n</textarea>"


def operate_on_generate_html(html_name: str) -> None:
    """Operate on generated HTML file to add styling and functionality.

    Args:
        html_name (str): Name of the HTML file to process.
    """
    if not (data_list := read_list(html_name, "r", None)):
        return None

    head_list = [html_head.format(os.path.basename(html_name).split(".")[0].strip()), html_style, "\n"]
    tail_list = [html_tail]

    content = "".join(data_list)
    content = content.replace("<pre><code>", textarea_header).replace("</code></pre>", textarea_tail)
    for i in re.findall(r"<li>(.*?)<details>", content, re.DOTALL):
        content = content.replace(rf"<li>{i}<details>", f"<li><details>\n<summary>\n{i.strip()}\n</summary>")
    data_list = combine_content_in_list([head_list, [content], tail_list])
    write_list(data_list, html_name, "w", None, False)
    return None


def is_last_week_of_month():
    """
    Check if today's date falls in the last week of the current month.

    Returns:
        bool: True if today is in the last week of the month, False otherwise.
    """
    # Get today's date
    today = datetime.date.today()

    # Calculate the last day of the current month
    # First, find the first day of next month
    if today.month == 12:
        next_month = datetime.date(today.year + 1, 1, 1)
    else:
        next_month = datetime.date(today.year, today.month + 1, 1)

    # Subtract one day to get the last day of the current month
    last_day_of_month = next_month - datetime.timedelta(days=1)

    # Calculate the week number of today and the last day of the month
    # Using isocalendar() which returns (year, week number, weekday)
    today_week = today.isocalendar()[1]
    last_day_week = last_day_of_month.isocalendar()[1]

    # For December, the week number might roll over to next year
    # If so, we need to adjust the comparison
    if today.month == 12 and today_week < last_day_week:
        # This handles the case where the last week of December is actually
        # the first week of the next year in the ISO calendar
        return True

    # Check if we're in the same week as the last day of the month
    return today_week == last_day_week



import copy
import os
from typing import Dict, List, Tuple

from pyadvtools import combine_content_in_list, read_list, write_list
from pybibtexer.bib.bibtexparser import Library
from pybibtexer.main import PythonWriters

from ...main import BasicInput, PandocMdTo
from ...tools.search.utils import combine_keywords_for_file_name, combine_keywords_for_title, keywords_type_for_title


class WriteInitialResult(BasicInput):
    """Write initial results for single keyword.

    Args:
        options (dict): Configuration options.

    Attributes:
        options (dict): Configuration options.
    """

    def __init__(self, options: dict) -> None:
        """Initialize WriteInitialResult with configuration options.

        Args:
            options (dict): Configuration options.
        """
        super().__init__(options)

        self._level_title_md = "###"
        self._level_title_tex = "subsection"
        self._pandoc_md_to = PandocMdTo(options)

    def main(
        self,
        path_initial: str,
        output_prefix: str,
        field: str,
        keywords_type: str,
        combine_keywords: str,
        library_for_abbr: Library,
        library_for_zotero: Library,
        library_for_save: Library,
    ) -> Tuple[List[List[str]], List[str]]:
        """Main method to write initial results.

        Args:
            path_initial (str): Path to initial directory.
            output_prefix (str): Prefix for output files.
            field (str): Field being searched.
            keywords_type (str): Type of keywords.
            combine_keywords (str): Combined keywords string.
            library_for_abbr (Library): Abbreviated bibliography library.
            library_for_zotero (Library): Zotero bibliography library.
            library_for_save (Library): Save bibliography library.

        Returns:
            Tuple[List[List[str]], List[str]]: Tuple containing data and error messages.
        """
        error_pandoc_md_md = []

        # generate
        cite_keys = [entry.key for entry in library_for_abbr.entries]

        # update options
        _options = copy.deepcopy(self.options)
        _options["keep_entries_by_cite_keys"] = cite_keys
        _python_writer = PythonWriters(_options)

        # generate tex and md data
        data_list_tex, data_list_md, header = self.generate_content_tex_md(
            cite_keys, output_prefix, field, combine_keywords
        )

        # definition
        file_prefix = combine_keywords_for_file_name(combine_keywords)  # the file name prefix

        # write tex, md, and bib files
        data_list = [data_list_tex, data_list_md, library_for_abbr, library_for_zotero, library_for_save]
        mid_list = ["", "", "-abbr", "-zotero", "-save"]
        post_list = ["tex", "md", "bib", "bib", "bib"]
        path_write = os.path.join(path_initial, f"{field}-{keywords_type}")
        for i in range(len(post_list)):
            file_name = f"{file_prefix}{mid_list[i]}.{post_list[i]}"
            _python_writer.write_to_file(data_list[i], file_name, "w", path_write)

        # pandoc md to generate md file
        path_bib = os.path.join(path_write, f"{file_prefix}{mid_list[2]}.bib")  # bib_for_abbr
        data_list_pandoc_md = self._pandoc_md_to.pandoc_md_to_md(
            path_bib, path_write, path_write, f"{file_prefix}.md", f"{file_prefix}-pandoc.md"
        )

        # mian part
        # generate some md output data
        data_basic_md: List[str] = []
        data_beauty_md: List[str] = []
        data_complex_md: List[str] = []
        if data_list_pandoc_md:
            data_basic_md, data_beauty_md, data_complex_md = self.generate_basic_beauty_complex_md(
                header, cite_keys, data_list_pandoc_md, library_for_zotero
            )
        else:
            error_pandoc_md_md.append(f"- pandoc full false: {file_prefix}_pandoc.md" + "\n")

        # write basic beauty complex md files
        basic_beauty_complex = ["-basic", "-beauty", "-complex"]
        for d, name in zip([data_basic_md, data_beauty_md, data_complex_md], basic_beauty_complex):
            write_list(d, "{}{}.md".format(file_prefix, name), "w", path_write)

        # save all (tex, md, bib) files
        x = [f"{i}.{j}" for i, j in zip(mid_list, post_list)]
        x.extend([f"{i}.md" for i in basic_beauty_complex])
        data_temp = [[os.path.join(path_write, file_prefix + i)] for i in x]
        return data_temp, error_pandoc_md_md

    def generate_basic_beauty_complex_md(
        self, header: str, cite_key_list: List[str], data_list_pandoc_md: List[str], library_for_zotero: Library
    ) -> Tuple[List[str], List[str], List[str]]:
        """Generate basic, beauty, and complex markdown content.

        Args:
            header (str): Header string for the content.
            cite_key_list (List[str]): List of citation keys.
            data_list_pandoc_md (List[str]): List of pandoc markdown data.
            library_for_zotero (Library): Zotero bibliography library.

        Returns:
            Tuple[List[str], List[str], List[str]]: Tuple containing basic, beauty, and complex markdown content.
        """
        data_basic_md, data_beauty_md, data_complex_md = [], [], []

        # library
        _options = copy.deepcopy(self.options)
        _python_writer = PythonWriters(_options)
        key_url_http_bib_dict = _python_writer.output_key_url_http_bib_dict(library_for_zotero)

        key_basic_dict, key_beauty_dict, key_complex_dict = self._pandoc_md_to.generate_key_data_dict(
            data_list_pandoc_md, key_url_http_bib_dict
        )

        if key_basic_dict and key_beauty_dict and key_complex_dict:
            data_basic_md, data_beauty_md, data_complex_md = [header + "\n"], [header + "\n"], [header + "\n"]
            for i in range(length := len(cite_key_list)):
                data_basic_md.extend(self._convert_to_special_list(key_basic_dict.get(cite_key_list[i], [])))
                data_beauty_md.extend(self._convert_to_special_list(key_beauty_dict.get(cite_key_list[i], [])))
                data_complex_md.extend(self._convert_to_special_list(key_complex_dict.get(cite_key_list[i], [])))
                if i < (length - 1):
                    data_basic_md.append("\n")
                    data_beauty_md.append("\n")
                    data_complex_md.append("\n")
        return data_basic_md, data_beauty_md, data_complex_md

    @staticmethod
    def _convert_to_special_list(data_list: List[str]) -> List[str]:
        """Convert data list to special formatted list.

        Args:
            data_list (List[str]): List of data strings.

        Returns:
            List[str]: Formatted list with proper indentation.
        """
        if len(data_list) > 0:
            data_list[0] = "- " + data_list[0]
        for j in range(len(data_list) - 1):
            if data_list[j][-1] == "\n":
                data_list[j + 1] = "  " + data_list[j + 1]
        return data_list

    def generate_content_tex_md(
        self, cite_key_list: List[str], output_prefix: str, field: str, combine_keywords: str
    ) -> Tuple[List[str], List[str], str]:
        """Generate LaTeX and markdown content.

        Args:
            cite_key_list (List[str]): List of citation keys.
            output_prefix (str): Prefix for output files.
            field (str): Field being searched.
            combine_keywords (str): Combined keywords string.

        Returns:
            Tuple[List[str], List[str], str]: Tuple containing LaTeX content, markdown content, and header.
        """
        c_k_f_t = combine_keywords_for_title(combine_keywords)

        number_references = len(cite_key_list)
        _title = f"{output_prefix} {field} contains {number_references} {c_k_f_t}"

        tex_header = f"\\{self._level_title_tex}" + "{" + _title + "}\n"
        tex_body = ["\\nocite{" + f"{c_k}" + "}\n" for c_k in cite_key_list]
        tex_tail = "\\printbibliography\n\n\\ifx \\clearPage \\undefined \\else \\clearpage \\fi\n"
        data_list_tex = combine_content_in_list([[tex_header], ["\n"], tex_body, ["\n"], [tex_tail]])

        md_header = f"{self._level_title_md}" + " " + _title + "\n"
        md_body = [r"- [@" + f"{c_k}" + "]\n" for c_k in cite_key_list]
        data_list_md = combine_content_in_list([[md_header], ["\n"], md_body])
        return data_list_tex, data_list_md, md_header


class WriteSeparateResult(object):
    """Write separate result for different keyword types."""

    def __init__(self) -> None:
        """Initialize WriteSeparateResult with title levels."""
        self._level_title_md = "##"
        self._level_title_tex = "section"

    def main(
        self, data_temp: List[List[str]], field: str, keywords_type: str, combine_keywords: str, path_separate: str
    ) -> None:
        """Main method to write separate results.

        Args:
            data_temp (List[List[str]]): List of data lists for different file types.
            field (str): Field being processed.
            keywords_type (str): Type of keywords.
            combine_keywords (str): Combined keywords string.
            path_separate (str): Path to separate directory.
        """
        k_t_f_t = keywords_type_for_title(keywords_type)
        _title = f"{field.title()} contains {k_t_f_t}"

        file_prefix = combine_keywords_for_file_name(combine_keywords)  # the file name prefix
        mid_list = ["", "", "-abbr", "-zotero", "-save", "-basic", "-beauty", "-complex"]
        post_list = ["tex", "md", "bib", "bib", "bib", "md", "md", "md"]

        len_data_temp = len(data_temp)  # len(data_temp) = len(mid_list) = len(post_list) = 8
        split_flag = mid_list.index("-abbr")

        for i in range(split_flag, len_data_temp):
            path_temp = os.path.join(path_separate, f"{keywords_type}", f"{field}-{post_list[i]}{mid_list[i]}")
            full_file = os.path.join(path_temp, rf"{file_prefix}.{post_list[i]}")
            temp_data_list = read_list(data_temp[i][0], "r", None)
            if not os.path.isfile(full_file):
                if post_list[i] == "md":
                    temp_data_list.insert(0, f"{self._level_title_md}" + " " + _title + "\n\n")
                elif post_list[i] == "tex":
                    temp_data_list.insert(0, f"\\{self._level_title_tex}" + "{" + _title + "}\n\n")
            else:
                temp_data_list.insert(0, "\n")
            write_list(temp_data_list, full_file, "a", None, False, False)  # Compulsory `a`
        return None


class WriteAbbrCombinedResults(object):
    """Write combined results for abbreviations (such as `TEVC`, `PNAS`).

    Args:
        options (dict): Configuration options.

    Attributes:
        options (dict): Configuration options.
        pandoc_md_basic_to_pdf (bool): Whether to convert basic markdown to PDF.
        pandoc_md_beauty_to_pdf (bool): Whether to convert beauty markdown to PDF.
        pandoc_md_complex_to_pdf (bool): Whether to convert complex markdown to PDF.
        pandoc_md_basic_to_html (bool): Whether to convert basic markdown to HTML.
        pandoc_md_beauty_to_html (bool): Whether to convert beauty markdown to HTML.
        pandoc_md_complex_to_html (bool): Whether to convert complex markdown to HTML.
    """

    def __init__(self, options: dict) -> None:
        """Initialize WriteAbbrCombinedResults with configuration options.

        Args:
            options (dict): Configuration options.
        """
        self.pandoc_md_basic_to_pdf: bool = options.get("pandoc_md_basic_to_pdf", False)
        self.pandoc_md_beauty_to_pdf: bool = options.get("pandoc_md_beauty_to_pdf", False)
        self.pandoc_md_complex_to_pdf: bool = options.get("pandoc_md_complex_to_pdf", False)
        self.pandoc_md_basic_to_html: bool = options.get("pandoc_md_basic_to_html", False)
        self.pandoc_md_beauty_to_html: bool = options.get("pandoc_md_beauty_to_html", False)
        self.pandoc_md_complex_to_html: bool = options.get("pandoc_md_complex_to_html", True)

        self._level_title_md = "##"
        self._level_title_tex = "section"
        self._pandoc_md_to = PandocMdTo(options)

    def main(
        self, search_field_list, keywords_type: str, field_data_dict: Dict[str, List[List[str]]], path_combine: str
    ) -> Tuple[List[str], List[str]]:
        """Main method to write combined results for abbreviations.

        Args:
            search_field_list: List of search fields.
            keywords_type (str): Type of keywords.
            field_data_dict (Dict[str, List[List[str]]]): Dictionary containing field data.
            path_combine (str): Path to combine directory.

        Returns:
            Tuple[List[str], List[str]]: Tuple containing error messages for PDF and HTML conversion.
        """
        path_subsection = os.path.join(path_combine, "tex-subsection")
        path_md = os.path.join(path_combine, "md")
        path_bib = os.path.join(path_combine, "bib")

        mid_list = ["", "", "-abbr", "-zotero", "-save", "-basic", "-beauty", "-complex"]
        post_list = ["tex", "md", "bib", "bib", "bib", "md", "md", "md"]
        path_list = [path_subsection, path_md, path_bib, path_bib, path_bib]
        for i in ["-basic", "-beauty", "-complex"]:
            path_list.append(os.path.join(path_combine, f"md{i}"))
        # len(mid_list) == len(post_list) == len(path_list) == 8

        k_t_f_t = keywords_type_for_title(keywords_type)

        error_pandoc_md_pdf, error_pandoc_md_html = [], []
        for field in search_field_list:
            if not field_data_dict.get(field):
                continue

            # write files
            file_prefix = f"{field}-{keywords_type}"  # the file name prefix
            _title = f"{field.title()} contains {k_t_f_t}"

            for j in range(0, len(post_list)):
                temp = combine_content_in_list([read_list(file, "r") for file in field_data_dict[field][j]], ["\n"])
                if post_list[j] == "md":
                    temp.insert(0, f"{self._level_title_md}" + " " + _title + "\n\n")
                elif post_list[j] == "tex":
                    temp.insert(0, f"\\{self._level_title_tex}" + "{" + _title + "}\n\n")
                write_list(temp, f"{file_prefix}{mid_list[j]}.{post_list[j]}", "w", path_list[j])

            # generate tex pdf html
            # for tex
            self._pandoc_md_to.generate_tex_content(file_prefix, path_subsection, path_bib, path_combine)

            # for pdf
            for i in ["basic", "beauty", "complex"]:
                if eval(f"self.pandoc_md_{i}_to_pdf"):
                    error_flag_pdf = self._pandoc_md_to.pandoc_md_to_pdf(
                        os.path.join(path_combine, f"md-{i}"),
                        f"{file_prefix}-{i}.md",
                        os.path.join(path_combine, f"pdf-{i}"),
                        f"{file_prefix}-{i}.pdf",
                    )
                    if error_flag_pdf:
                        error_pandoc_md_pdf.append(error_flag_pdf)

            # for html
            for i in ["basic", "beauty", "complex"]:
                if eval(f"self.pandoc_md_{i}_to_html"):
                    error_flag_html = self._pandoc_md_to.pandoc_md_to_html(
                        os.path.join(path_combine, f"md-{i}"),
                        os.path.join(path_combine, f"html-{i}"),
                        f"{file_prefix}-{i}.md",
                        f"{file_prefix}-{i}.html",
                        True,
                    )
                    if error_flag_html:
                        error_pandoc_md_html.append(error_flag_html)
        return error_pandoc_md_pdf, error_pandoc_md_html
